LayerOutput<N extends num, E, T extends Signal<N, E, T>, S extends Scale<N>> class

Layer specialized for output neurons.

Inheritance

Properties

activationFunction → ActivationFunction<N, E>
finalinherited
annRef ↔ ANN<N, E, T, S>?
Back-reference to the owning ANN (set during build); used by dropout to read ANN.trainingMode.
getter/setter pairinherited
asLayerHidden → LayerHidden<N, E, T, S>
no setterinherited
asLayerInput → LayerInput<N, E, T, S>
no setterinherited
asLayerOutput → LayerOutput<N, E, T, S>
no setteroverride
format → String
The signal format of this Layer.
no setterinherited
gradients → List<T>
no setterinherited
hashCode → int
The hash code for this object.
no setterinherited
hasNextLayer → bool
no setterinherited
layerType → String
no setteroverride
length → int
no setterinherited
neurons → T
no setterinherited
nextLayer → Layer<N, E, T, S>?
no setterinherited
previousGradients → List<T>
no setterinherited
previousLayer → Layer<N, E, T, S>?
no setterinherited
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited
weights → List<T>
no setterinherited
withBiasNeuron → bool
finalinherited

Methods

activateLayer() → void
override
connectTo(Layer<N, E, T, S> nextLayer, {Random? random, List<T>? weights}) → void
inherited
getNeurons() → List<N>
inherited
getNeuronsAsDouble() → List<double>
inherited
getNeuronsAsString() → List<String>
inherited
getNeuronsDenormalized(S scale) → List<N>
inherited
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
resetGradients() → void
inherited
resetLayer() → void
inherited
resetNetwork() → void
inherited
resetNextLayerNetwork() → void
inherited
resetWeights([Random? rand]) → void
inherited
setNeurons(T signal) → void
inherited
toJson({bool withIndent = false}) → String
Converts to an encoded JSON.
inherited
toJsonMap() → Map<String, dynamic>
Converts this Layer to a JSON Map.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited