ScaleDouble class
A Scale<double>.
Constructors
- ScaleDouble(double minValue, double maxValue)
Properties
- format → String
-
The data format of this scale.
no setteroverride
- hashCode → int
-
The hash code for this object.
no setterinherited
- maxValue → double
-
finalinherited
- minValue → double
-
finalinherited
- range ↔ double
-
latefinalinherited
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
- zero → double
-
The
zerovalue for this scale format.final
Methods
-
denormalize(
double normalizedValue) → double -
Denormalize
normalizedValueto values of this scale (in the range minValue to maxValue).override -
denormalizeList(
List< double> normalizedValues) → List<double> -
Denormalizes
valuesusing denormalize.inherited -
normalize(
double value) → double -
Normalize
valueto this scale (in the range0..1)override -
normalizeList(
List< double> values) → List<double> -
Normalizes
valuesusing normalize.inherited -
normalizeNum(
num value) → double -
Normalize
valueto this scale (in the range0..1)override -
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
toJson(
{bool withIndent = false}) → String -
Converts to an encoded JSON.
inherited
-
toJsonMap(
) → Map< String, dynamic> -
Converts this Scale to a JSON Map.
inherited
-
toN(
num value) → double -
Converts
valuetoN.override -
toString(
) → String -
A string representation of this object.
override
Operators
-
operator ==(
Object other) → bool -
Returns
trueifotheris equals tothis.inherited
Static Properties
- ZERO_TO_ONE → ScaleDouble
-
final