LearningRateScheduleStrategy<N extends num, E, T extends Signal<N, E, T> > class
abstract
Base for epoch-based learning-rate schedules. The value is recomputed each epoch from Training.trainedEpochs via computeValue.
- Inheritance
-
- Object
- ParameterStrategy<
N, E, T> - LearningRateScheduleStrategy
- Implementers
Constructors
-
LearningRateScheduleStrategy(Propagation<
N, E, T, dynamic, dynamic> propagation, double baseValue)
Properties
- baseValue → double
-
final
- hashCode → int
-
The hash code for this object.
no setterinherited
- initialValue → double
-
no setteroverride
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
- value → double
-
no setteroverride
- valueEntry → E
-
no setteroverride
Methods
-
computeValue(
int epoch) → double -
The scheduled value at
epoch(0-based). -
createValueEntry(
double value) → E -
inherited
-
initializeValue(
) → void -
override
-
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
resetValue(
) → void -
override
-
setValue(
double value) → void -
override
-
toString(
) → String -
A string representation of this object.
inherited
-
updateValue(
) → void -
override
Operators
-
operator ==(
Object other) → bool -
The equality operator.
inherited